@@ -1,9 +1,3 @@
-/*
- * To change this license header, choose License Headers in Project Properties.
- * To change this template file, choose Tools | Templates
- * and open the template in the editor.
- */
-
 /* 
  * File:   biconnected.h
  * Author: mb03
@@ -23,17 +17,27 @@ extern "C" {
         struct graph g;
         int * mapping;
         int * weights;
-        
-        bool added;
-        
         struct node_graph * cutpoint;
         int cutpoint_index;
     };
     
+    struct sub_graph{
+        struct list connected_components;
+        int size;
+    };
+    
+    //These function returns the list of list of connected components
+    //i.e. a list of connected components for each connected subgraph
+    struct list*  tarjan_rec_undir(struct graph * g, 
+            bool * is_articulation_point, int * component_indexes);
+    struct list*  tarjan_iter_undir(struct graph * g, 
+            bool * is_articulation_point, int * component_indexes);
+    
+    //not employed  and completed yet (missing art_poit and 
+    //division in subgraph with componenent indexing)
     struct list*  tarjan_rec_dir(struct graph * g);
     struct list*  tarjan_iter_dir(struct graph * g);
-    struct list*  tarjan_rec_undir(struct graph * g, bool * is_articulation_point);
-    struct list*  tarjan_iter_undir(struct graph * g, bool * is_articulation_point);
+    
     
 #ifdef __cplusplus
 }
