@@ -1,9 +1,3 @@
-/*
- * To change this license header, choose License Headers in Project Properties.
- * To change this template file, choose Tools | Templates
- * and open the template in the editor.
- */
-
 /* 
  * File:   graph.h
  * Author: mb03
@@ -20,18 +14,14 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
-    
+    /**
+     * Structs and functions used to represent a graph. Further detail in grap.c
+     */
     
     struct node_graph {
         struct list neighbours;
         char * name;
-        //for recursive version
-       // int index;
-        //int low_link;
-       // bool on_stack;
-       // int bcc_id;
-
-        int node_graph_id; //will be replaced by index
+        int node_graph_id;
     }; 
     struct edge_graph {
         struct node_graph * to;
@@ -43,15 +33,11 @@ extern "C" {
     };
     
     void init_graph(struct graph * g);
-    struct node_graph * add_node_graph(struct graph * g, const char * name);
     void add_edge_graph(struct graph * g, const char * name_from, const char * name_to, double value, bool directed);
     void add_edge_graph_return_node_indexes(struct graph * g, const char * name_from, const char * name_to, double value, bool directed,int  * nodefrom, int * nodeto);
-    void print_graph(struct graph * g);
     void free_graph(struct graph * g);
     
-    void init_node_graph(struct node_graph * n,const char * name, int node_graph_id);
-    void init_edge_graph(struct edge_graph * e);
-    void init_edge_graph_params(struct edge_graph * e,struct node_graph * to,double value);
+
     
     
 #ifdef __cplusplus
