@@ -1,10 +1,3 @@
-/*
- * parser.h
- *
- *  Created on: 02 lug 2016
- *      Author: gabriel
- */
-
 #ifndef SRC_PARSER_H_
 #define SRC_PARSER_H_
 
@@ -50,14 +43,15 @@ struct neighbor{
 	struct neighbor *next;
 };
 
-
+struct topology * parse_jsoninfo(char *buffer);
 struct topology * parse_netjson(char* buffer);
 int add_node(struct topology * topo, const char *id);
 struct topology * _init_topo(int type);
 int add_neigh(struct topology *topo, const char *source, const char *id, const double weight);
-
+void destroy_topo(struct topology *topo);
 struct node* find_node(struct topology *topo, const char *id);
 void bc_degree_map_delete(map_id_degree_bc * map);
 
+void destroy_topo(struct topology *topo);
 
 #endif /* SRC_PARSER_H_ */
